@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2003 the xine project
+ * Copyright (C) 2000-2004 the xine project
  *
  * This file is part of xine, a unix video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: main.c,v 1.275 2003/12/23 16:05:15 siggi Exp $
+ * $Id: main.c,v 1.283 2004/04/15 19:07:42 f1rmb Exp $
  *
  * xine main for X11
  *
@@ -269,6 +269,17 @@ static char **build_command_line_args(int argc, char *argv[], int *_argc) {
   return _argv;
 }
 
+static void free_command_line_args(char ***argv, int argc) {
+  int _argc = argc - 1;
+  
+  while(_argc >= 0) {
+    free((*argv)[_argc]);
+    _argc--;
+  }
+  
+  free(*(argv));
+}
+
 static int parse_geometry(window_attributes_t *window_attribute, char *geomstr) {
   int width, height, xoff, yoff, ret;  
   
@@ -991,17 +1002,6 @@ static void event_listener(void *user_data, const xine_event_t *event) {
   
   switch(event->type) { 
     
-  case XINE_EVENT_SPU_BUTTON:
-    {
-      xine_spu_button_t *spubtn = (xine_spu_button_t *) event->data;
-
-      if(spubtn->direction)
-	video_window_set_cursor(CURSOR_HAND);
-      else
-	video_window_set_cursor(CURSOR_ARROW);
-    }
-    break;
-
   /* frontend can e.g. move on to next playlist entry */
   case XINE_EVENT_UI_PLAYBACK_FINISHED:
     if(event->stream == gGui->stream) {
@@ -1094,108 +1094,115 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 	/* (warning message) */
       case XINE_MSG_GENERAL_WARNING:
 	report = xine_info;
-	sprintf(buffer, "%s:", _("*drum roll*, xine lib wants to take your attention "
-				 "to deliver an important message to you ;-):"));
+	snprintf(buffer, sizeof(buffer),
+		 "%s:", _("*drum roll*, xine lib wants to take your attention "
+			  "to deliver an important message to you ;-):"));
 	if(data->explanation)
-	  sprintf(buffer, "%s %s %s", buffer,
-		  (char *) data + data->explanation, (char *) data + data->parameters);
+	  snprintf(buffer, sizeof(buffer), "%s %s %s", buffer,
+		   (char *) data + data->explanation, (char *) data + data->parameters);
 	else
-	  sprintf(buffer, "%s %s", buffer, _("No Information available."));
+	  snprintf(buffer, sizeof(buffer), "%s %s", buffer, _("No Information available."));
 	  
 	break;
 
         /* (host name) */
       case XINE_MSG_UNKNOWN_HOST:
-	sprintf(buffer, "%s", _("The host you're trying to connect is unknown.\n"
-				"Check the validity of the specified hostname."));
+	snprintf(buffer, sizeof(buffer), "%s", _("The host you're trying to connect is unknown.\n"
+						 "Check the validity of the specified hostname."));
 	if(data->explanation)
-	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
+	  snprintf(buffer, sizeof(buffer), "%s (%s)", buffer, (char *) data + data->parameters);
 	break;
 	
 	/* (device name) */
       case XINE_MSG_UNKNOWN_DEVICE:
-	sprintf(buffer, "%s", _("The device name you specified seems invalid."));
+	snprintf(buffer, sizeof(buffer), "%s", _("The device name you specified seems invalid."));
 	if(data->explanation)
-	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
+	  snprintf(buffer, sizeof(buffer), "%s (%s)", buffer, (char *) data + data->parameters);
 	break;
 
 	/* none */
       case XINE_MSG_NETWORK_UNREACHABLE:
-	sprintf(buffer, "%s", _("The network looks unreachable.\nCheck your network "
-				"setup and/or the server name."));
+	snprintf(buffer, sizeof(buffer), "%s", _("The network looks unreachable.\nCheck your network "
+						 "setup and/or the server name."));
 	if(data->explanation)
-	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
+	  snprintf(buffer, sizeof(buffer), "%s (%s)", buffer, (char *) data + data->parameters);
 	break;
 
 	/* (host name) */
       case XINE_MSG_CONNECTION_REFUSED:
-	sprintf(buffer, "%s", _("The connection was refused.\nCheck the host name."));
+	snprintf(buffer, sizeof(buffer), "%s", _("The connection was refused.\nCheck the host name."));
 	if(data->explanation)
-	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
+	  snprintf(buffer, sizeof(buffer), "%s (%s)", buffer, (char *) data + data->parameters);
 	break;
 
 	/* (file name or mrl) */
       case XINE_MSG_FILE_NOT_FOUND:
-	sprintf(buffer, "%s", _("The specified file or mrl is not found. Please check it twice."));
+	snprintf(buffer, sizeof(buffer), "%s", _("The specified file or mrl is not found. Please check it twice."));
 	if(data->explanation)
-	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
+	  snprintf(buffer, sizeof(buffer), "%s (%s)", buffer, (char *) data + data->parameters);
 	break;
 
 	/* (device/file/mrl) */
       case XINE_MSG_READ_ERROR:
-	sprintf(buffer, "%s", _("The source can't be read.\nMaybe you don't have enough "
-				"rights for this, or source doesn't contain data "
-				"(e.g: not disc in drive)."));
+	snprintf(buffer, sizeof(buffer), "%s", _("The source can't be read.\nMaybe you don't have enough "
+						 "rights for this, or source doesn't contain data "
+						 "(e.g: not disc in drive)."));
 	if(data->explanation)
-	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
+	  snprintf(buffer, sizeof(buffer), "%s (%s)", buffer, (char *) data + data->parameters);
 	break;
 	
 	/* (library/decoder) */
       case XINE_MSG_LIBRARY_LOAD_ERROR:
-	sprintf(buffer, "%s", _("A problem occur while loading a library or a decoder"));
+	snprintf(buffer, sizeof(buffer), "%s", _("A problem occur while loading a library or a decoder"));
 	if(data->explanation)
-	  sprintf(buffer, "%s: %s", buffer, (char *) data + data->parameters);
+	  snprintf(buffer, sizeof(buffer), "%s: %s", buffer, (char *) data + data->parameters);
 	break;
 
 	/* none */
       case XINE_MSG_ENCRYPTED_SOURCE:
-	sprintf(buffer, "%s", _("The source seems encrypted, and can't be read."));
+	snprintf(buffer, sizeof(buffer), "%s", _("The source seems encrypted, and can't be read."));
 	if(!strncasecmp(gGui->mmk.mrl, "dvd:/", 5)) {
-	  sprintf(buffer, "%s", _("\nYou DVD is probably crypted. "
-				  "According to your country laws, you can or can't "
-				  "install/use libdvdcss to be able to read this disc, "
-				  "which you bought."));
+	  snprintf(buffer, sizeof(buffer), "%s", _("\nYou DVD is probably crypted. "
+						   "According to your country laws, you can or can't "
+						   "install/use libdvdcss to be able to read this disc, "
+						   "which you bought."));
 	}
 	if(data->explanation)
-	  sprintf(buffer, "%s (%s)", buffer, (char *) data + data->parameters);
+	  snprintf(buffer, sizeof(buffer), "%s (%s)", buffer, (char *) data + data->parameters);
 	break;
 
 	/* (warning message) */
       case XINE_MSG_SECURITY:
 	report = xine_info;
 	if(data->explanation)
-	  sprintf(buffer, "%s %s",
-		  (char *) data + data->explanation, (char *) data + data->parameters);
+	  snprintf(buffer, sizeof(buffer), "%s %s",
+		   (char *) data + data->explanation, (char *) data + data->parameters);
 	else
-	  sprintf(buffer, _("No Informations available."));
+	  snprintf(buffer, sizeof(buffer), "%s", _("No Informations available."));
+	break;
+
+      case XINE_MSG_AUDIO_OUT_UNAVAILABLE:
+	gui_stop(NULL, NULL);
+	snprintf(buffer, sizeof(buffer), "%s", _("The audio device is unavailable. "
+						 "Please verify if another program already use it."));
 	break;
 
       default:
-	sprintf(buffer, "%s", _("*sight*, unkown error."));
+	snprintf(buffer, sizeof(buffer), "%s", _("*sight*, unkown error."));
 	if(data->explanation)
-	  sprintf(buffer, "%s (%s %s)", buffer,
-		  (char *) data + data->explanation, (char *) data + data->parameters);
+	  snprintf(buffer, sizeof(buffer), "%s (%s %s)", buffer,
+		   (char *) data + data->explanation, (char *) data + data->parameters);
 	break;
       }
       
       if(gGui->verbosity >= XINE_VERBOSITY_DEBUG) {
-	sprintf(buffer, "%s\n\n[", buffer);
+	snprintf(buffer, sizeof(buffer), "%s\n\n[", buffer);
 	
 	if(data->explanation)
-	  sprintf(buffer, "%s'%s' '%s'", buffer, 
-		  (char *) data + data->explanation, (char *) data + data->parameters);
+	  snprintf(buffer, sizeof(buffer), "%s'%s' '%s'", buffer, 
+		   (char *) data + data->explanation, (char *) data + data->parameters);
 	
-	sprintf(buffer, "%s]", buffer);
+	snprintf(buffer, sizeof(buffer), "%s]", buffer);
       }
       
       if(strlen(buffer))
@@ -1271,6 +1278,31 @@ static void event_listener(void *user_data, const xine_event_t *event) {
 
     }
     break;
+
+  case XINE_EVENT_UI_NUM_BUTTONS:
+    break;
+
+  case XINE_EVENT_SPU_BUTTON:
+    {
+      xine_spu_button_t *spubtn = (xine_spu_button_t *) event->data;
+      
+      if(spubtn->direction)
+	video_window_set_cursor(CURSOR_HAND);
+      else
+	video_window_set_cursor(CURSOR_ARROW);
+    }
+    break;
+
+  case XINE_EVENT_DROPPED_FRAMES:
+    {
+      char buffer[2048];
+      
+      memset(&buffer, 0, sizeof(buffer));
+      snprintf(buffer, sizeof(buffer), "%s", _("Your system seems too slow, the amoung of dropped frames is too high."));
+      xine_error_with_more(buffer);
+    }
+    break;
+
   }
 }
   
@@ -1417,24 +1449,30 @@ int main(int argc, char *argv[]) {
       break;
 
     case 'p':/* Play [[in fullscreen][then quit]] on start */
-      gGui->actions_on_start[aos++] = ACTID_PLAY;
+      if(aos < MAX_ACTIONS_ON_START)
+	gGui->actions_on_start[aos++] = ACTID_PLAY;
       if(optarg != NULL) {
 	if(strrchr(optarg, 'f')) {
-	  gGui->actions_on_start[aos++] = ACTID_TOGGLE_FULLSCREEN;
+	  if(aos < MAX_ACTIONS_ON_START)
+	    gGui->actions_on_start[aos++] = ACTID_TOGGLE_FULLSCREEN;
 	}
 #ifdef HAVE_XINERAMA
 	if(strrchr(optarg, 'F')) {
-	  gGui->actions_on_start[aos++] = ACTID_TOGGLE_XINERAMA_FULLSCREEN;
+	  if(aos < MAX_ACTIONS_ON_START)
+	    gGui->actions_on_start[aos++] = ACTID_TOGGLE_XINERAMA_FULLSCREEN;
 	}
 #endif
 	if(strrchr(optarg, 'h')) {
-	  gGui->actions_on_start[aos++] = ACTID_TOGGLE_VISIBLITY;
+	  if(aos < MAX_ACTIONS_ON_START)
+	    gGui->actions_on_start[aos++] = ACTID_TOGGLE_VISIBLITY;
 	}
 	if(strrchr(optarg, 'q')) {
-	  gGui->actions_on_start[aos++] = ACTID_QUIT;
+	  if(aos < MAX_ACTIONS_ON_START)
+	    gGui->actions_on_start[aos++] = ACTID_QUIT;
 	}
 	if(strrchr(optarg, 'w')) {
-	  gGui->actions_on_start[aos++] = ACTID_TOGGLE_WINOUT_VISIBLITY;
+	  if(aos < MAX_ACTIONS_ON_START)
+	    gGui->actions_on_start[aos++] = ACTID_TOGGLE_WINOUT_VISIBLITY;
 	}
 	if(strrchr(optarg, 'd')) {
 	  gGui->autoscan_plugin = "DVD";
@@ -1446,20 +1484,24 @@ int main(int argc, char *argv[]) {
       break;
 
     case 'g': /* hide panel on start */
-      gGui->actions_on_start[aos++] = ACTID_TOGGLE_VISIBLITY;
+      if(aos < MAX_ACTIONS_ON_START)
+	gGui->actions_on_start[aos++] = ACTID_TOGGLE_VISIBLITY;
       break;
 
     case 'H': /* hide video on start */
-      gGui->actions_on_start[aos++] = ACTID_TOGGLE_WINOUT_VISIBLITY;
+      if(aos < MAX_ACTIONS_ON_START)
+	gGui->actions_on_start[aos++] = ACTID_TOGGLE_WINOUT_VISIBLITY;
       break;
 
     case 'f': /* full screen mode on start */
-      gGui->actions_on_start[aos++] = ACTID_TOGGLE_FULLSCREEN;
+      if(aos < MAX_ACTIONS_ON_START)
+	gGui->actions_on_start[aos++] = ACTID_TOGGLE_FULLSCREEN;
       break;
 
 #ifdef HAVE_XINERAMA
     case 'F': /* xinerama full screen mode on start */
-      gGui->actions_on_start[aos++] = ACTID_TOGGLE_XINERAMA_FULLSCREEN;
+      if(aos < MAX_ACTIONS_ON_START)
+	gGui->actions_on_start[aos++] = ACTID_TOGGLE_XINERAMA_FULLSCREEN;
       break;
 #endif
 
@@ -1568,8 +1610,10 @@ int main(int argc, char *argv[]) {
       break;
 
     case 'P':
-      gGui->actions_on_start[aos++] = ACTID_PLAYLIST;
-      gGui->playlist.on_start = strdup(optarg);
+      if(aos < MAX_ACTIONS_ON_START) {
+	gGui->actions_on_start[aos++] = ACTID_PLAYLIST;
+	gGui->playlist.on_start = strdup(optarg);
+      }
       break;
 
     case 'l':
@@ -1745,8 +1789,10 @@ int main(int argc, char *argv[]) {
 	  sprintf(session_argv[session_argv_num], "mrl=%s", p);
 	  session_argv[++session_argv_num]   = NULL;
 
-	  gGui->actions_on_start[aos++] = ACTID_PLAY;
-	  gGui->actions_on_start[aos++] = ACTID_QUIT;
+	  if(aos < MAX_ACTIONS_ON_START)
+	    gGui->actions_on_start[aos++] = ACTID_PLAY;
+	  if(aos < MAX_ACTIONS_ON_START)
+	    gGui->actions_on_start[aos++] = ACTID_QUIT;
 	}
       }
       break;
@@ -1767,8 +1813,8 @@ int main(int argc, char *argv[]) {
       if(_argv[optind]) {
 	for(i = optind; i < _argc; i++) {
 	  char enqueue_mrl[strlen(_argv[i]) + 256];
-	  
-	  sprintf(enqueue_mrl, "session=%d,mrl=%s", session, _argv[i]);
+
+	  sprintf(enqueue_mrl, "session=%d,mrl=%s", session, atoa(_argv[i]));
 	  session_handle_subopt(enqueue_mrl, &session);
 	}
       }
@@ -1777,6 +1823,7 @@ int main(int argc, char *argv[]) {
 
     }
 
+    free_command_line_args(&_argv, _argc);
     return 0;
   }
 
@@ -1807,8 +1854,10 @@ int main(int argc, char *argv[]) {
     sprintf (gGui->configfile + strlen(gGui->configfile), "/%s", cfgfile);
     
     /* Popup setup window if there is no config file */
-    if(stat(gGui->configfile, &st) < 0)
-      gGui->actions_on_start[aos++] = ACTID_SETUP;
+    if(stat(gGui->configfile, &st) < 0) {
+      if(aos < MAX_ACTIONS_ON_START)
+	gGui->actions_on_start[aos++] = ACTID_SETUP;
+    }
     
   }
   
@@ -1871,8 +1920,10 @@ int main(int argc, char *argv[]) {
 			        NULL,
 			        CONFIG_NO_DATA);
 
-  if( enable_deinterlace )
-    gGui->actions_on_start[aos++] = ACTID_TOGGLE_INTERLEAVE;
+  if( enable_deinterlace ) {
+    if(aos < MAX_ACTIONS_ON_START)
+      gGui->actions_on_start[aos++] = ACTID_TOGGLE_INTERLEAVE;
+  }
 
   /*
    * init gui
@@ -1884,14 +1935,16 @@ int main(int argc, char *argv[]) {
   /* Automatically start playback if new_mode is enabled and playlist is filled */
   if((gGui->smart_mode && 
       (gGui->playlist.num || actions_on_start(gGui->actions_on_start, ACTID_PLAYLIST)) &&
-      (!(actions_on_start(gGui->actions_on_start, ACTID_PLAY)))) && (no_auto_start == 0))
-    gGui->actions_on_start[aos++] = ACTID_PLAY;
+      (!(actions_on_start(gGui->actions_on_start, ACTID_PLAY)))) && (no_auto_start == 0)) {
+    if(aos < MAX_ACTIONS_ON_START)
+      gGui->actions_on_start[aos++] = ACTID_PLAY;
+  }
   
   /*
    * xine init
    */
   xine_init(gGui->xine);
-  
+
   /* Get old working path from input plugin */
   {
     xine_cfg_entry_t  cfg_entry;
@@ -1947,12 +2000,15 @@ int main(int argc, char *argv[]) {
 	  if(((!strcmp(cfg_entry.enum_values[cfg_entry.num_value], "letterboxed tv")) ||
 	      (!strcmp(cfg_entry.enum_values[cfg_entry.num_value], "widescreen tv"))) && 
 	     (!(actions_on_start(gGui->actions_on_start, ACTID_TOGGLE_WINOUT_VISIBLITY)))) {
-	    gGui->actions_on_start[aos++] = ACTID_TOGGLE_WINOUT_VISIBLITY;
+	    if(aos < MAX_ACTIONS_ON_START)
+	      gGui->actions_on_start[aos++] = ACTID_TOGGLE_WINOUT_VISIBLITY;
 	  }
 	}
       }
-      else if(!strcasecmp(video_driver_ids[cfg_vo_entry.num_value], "fb"))
-	gGui->actions_on_start[aos++] = ACTID_TOGGLE_WINOUT_VISIBLITY;
+      else if(!strcasecmp(video_driver_ids[cfg_vo_entry.num_value], "fb")) {
+	if(aos < MAX_ACTIONS_ON_START)
+	  gGui->actions_on_start[aos++] = ACTID_TOGGLE_WINOUT_VISIBLITY;
+      }
 
     }
   }
@@ -1993,6 +2049,9 @@ int main(int argc, char *argv[]) {
   post_init();
 
   gGui->stream = xine_stream_new(gGui->xine, gGui->ao_port, gGui->vo_port);
+  
+  gGui->vo_none = xine_open_video_driver(gGui->xine, "none", XINE_VISUAL_TYPE_NONE, NULL);
+  gGui->ao_none = xine_open_audio_driver(gGui->xine, "none", NULL);
 
   osd_init();
 
@@ -2033,11 +2092,6 @@ int main(int argc, char *argv[]) {
   xine_set_param(gGui->visual_anim.stream, XINE_PARAM_SPU_CHANNEL, -2);
   xine_set_param(gGui->visual_anim.stream, XINE_PARAM_AUDIO_REPORT_LEVEL, 0);
 
-  /* Playlist scanning feature stream */
-  gGui->playlist.scan_stream = xine_stream_new(gGui->xine, gGui->ao_port, gGui->vo_port);
-  xine_set_param(gGui->playlist.scan_stream, XINE_PARAM_AUDIO_REPORT_LEVEL, 0);
-  xine_set_param(gGui->playlist.scan_stream, XINE_PARAM_SPU_CHANNEL, -2);
-
   /* subtitle stream */
   gGui->spu_stream = xine_stream_new(gGui->xine, NULL, gGui->vo_port);
   xine_set_param(gGui->spu_stream, XINE_PARAM_AUDIO_REPORT_LEVEL, 0);
@@ -2050,7 +2104,8 @@ int main(int argc, char *argv[]) {
   /*
    * hand control over to gui
    */
-  gGui->actions_on_start[aos] = ACTID_NOKEY;
+  if(aos < MAX_ACTIONS_ON_START)
+    gGui->actions_on_start[aos] = ACTID_NOKEY;
   
   /* Initialize posts, if required */
   if(pplugins_num) {
@@ -2081,8 +2136,10 @@ int main(int argc, char *argv[]) {
   pthread_mutex_destroy(&gGui->xe_mutex);
   pthread_mutex_destroy(&gGui->download_mutex);
 
-  if(gGui->report)
+  if(gGui->report != stdout)
     fclose(gGui->report);
 
+  free_command_line_args(&_argv, _argc);
+
   return 0;
 }
